﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

namespace DvdInfoFix
{
    class DvdInfoFix
    {
        static void Main(string[] args)
        {
            String directory = ".";
            if (args.Length == 1) directory = args[0];
            else
            {
                Console.WriteLine();
                Console.WriteLine("DvdInfoFix by Marcus Handte");
                Console.WriteLine();
                Console.WriteLine("Sets the encoding to ISO-8859-1 and removes image references.");
                Console.WriteLine();
                Console.WriteLine("Usage DvdInfoFix [file|folder]");
                Console.WriteLine("   file: Changes a single file.");
                Console.WriteLine("   folder: Changes all .xml files in the folder.");
                Console.WriteLine();
                return;
            }
            List<FileInfo> files = new List<FileInfo>();
            if (File.Exists(directory))
                files.Add(new FileInfo(directory));
            else if (Directory.Exists(directory))
            {
                string[] names = Directory.GetFiles(directory, "*.xml");
                for (int i = 0; i < names.Length; i++)
                {
                    files.Add(new FileInfo(names[i]));
                }
            }
            foreach (FileInfo file in files) {
                try
                {
                    Process(file);
                }
                catch (IOException e)
                {
                    Console.WriteLine(e);
                }
            }
        }


        static void Process(FileInfo file)
        {
            Console.WriteLine("Processing: " + file.FullName);
            // move the file for backup
            string input = file.FullName + ".backup";
            if (File.Exists(input))
            {
                Console.WriteLine("--- Skipping " + file.Name + " due to existing .backup file.");
                return;
            }
            File.Move(file.FullName, input);
            // read the data from the file
            XmlTextReader reader = new XmlTextReader(new StreamReader(input, Encoding.GetEncoding("ISO-8859-1")));
            string dvdid = file.Name.Substring(0, file.Name.Length - 4);
            string dvdtitle = null;
            string studio = null;
            string leadperformer = null;
            string director = null;
            string mpaarating = null;
            string releasedate = null;
            string genre = null;
            string dataprovider = null;
            string duration = null;
            string synopsis = null;
            while (reader.Read())
            {
                if (reader.NodeType == XmlNodeType.Element)
                {
                    if (dvdtitle == null && reader.Name.Equals("dvdtitle", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            dvdtitle = reader.Value.Trim();
                    if (studio == null && reader.Name.Equals("studio", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            studio = reader.Value.Trim();
                    if (leadperformer == null && reader.Name.Equals("leadperformer", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            leadperformer = reader.Value.Trim();
                    if (director == null && reader.Name.Equals("director", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            director = reader.Value.Trim();
                    if (mpaarating == null && reader.Name.Equals("mpaarating", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            mpaarating = reader.Value.Trim();
                    if (releasedate == null && reader.Name.Equals("releasedate", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            releasedate = reader.Value.Trim();
                    if (dataprovider == null && reader.Name.Equals("dataprovider", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            dataprovider = reader.Value.Trim();
                    if (duration == null && reader.Name.Equals("duration", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            duration = reader.Value.Trim();
                    if (synopsis == null && reader.Name.Equals("synopsis", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            synopsis = reader.Value.Trim();
                    if (genre == null && reader.Name.Equals("genre", System.StringComparison.CurrentCultureIgnoreCase))
                        if (reader.Read() && reader.NodeType == XmlNodeType.Text)
                            genre = reader.Value.Trim();   
                }
            }
            reader.Close();
            // generate some debug output and encode the strings
            if (dvdtitle == null || dvdtitle.Length == 0) Console.WriteLine("--- No dvdTitle in " + file.Name);
            else dvdtitle = System.Security.SecurityElement.Escape(dvdtitle);
            if (studio == null || studio.Length == 0) Console.WriteLine("--- No studio in " + file.Name + " (" + dvdtitle + ")");
            else studio = System.Security.SecurityElement.Escape(studio);
            if (leadperformer == null || leadperformer.Length == 0) Console.WriteLine("--- No leadPerformer in " + file.Name + " (" + dvdtitle + ")");
            else leadperformer = System.Security.SecurityElement.Escape(leadperformer);
            if (director == null || director.Length == 0) Console.WriteLine("--- No director in " + file.Name + " (" + dvdtitle + ")");
            else director = System.Security.SecurityElement.Escape(director);
            if (mpaarating == null || mpaarating.Length == 0) Console.WriteLine("--- No MPAARating in " + file.Name + " (" + dvdtitle + ")");
            else mpaarating = System.Security.SecurityElement.Escape(mpaarating);
            if (releasedate == null || releasedate.Length == 0) Console.WriteLine("--- No releaseDate in " + file.Name + " (" + dvdtitle + ")");
            else releasedate = System.Security.SecurityElement.Escape(releasedate);
            if (genre == null || genre.Length == 0) Console.WriteLine("--- No genre in " + file.Name + " (" + dvdtitle + ")");
            else genre = System.Security.SecurityElement.Escape(genre);
            if (dataprovider == null || dataprovider.Length == 0) Console.WriteLine("--- No dataProvider in " + file.Name + " (" + dvdtitle + ")");
            else dataprovider = System.Security.SecurityElement.Escape(dataprovider);
            if (duration == null || duration.Length == 0) Console.WriteLine("--- No duration in " + file.Name + " (" + dvdtitle + ")");
            else duration = System.Security.SecurityElement.Escape(duration);
            if (synopsis == null || synopsis.Length == 0) Console.WriteLine("--- No synopsis in " + file.Name + " (" + dvdtitle + ")");
            else synopsis = System.Security.SecurityElement.Escape(synopsis);
            // fix common mistakes in leadPerformer and director field
            if (leadperformer != null)
            {
                leadperformer = leadperformer.Replace(",", ";");
                if (leadperformer.EndsWith(";"))
                    leadperformer = leadperformer.Substring(0, leadperformer.Length - 1);
            }
            if (director != null)
            {
                director = director.Replace(",", ";");
                if (director.EndsWith(";"))
                    director = director.Substring(0, director.Length - 1);
            }
            // write the output file
            FileStream output = File.Open(file.FullName, FileMode.CreateNew, FileAccess.Write, FileShare.None);
            StreamWriter w = new StreamWriter(output, Encoding.GetEncoding("ISO-8859-1"));
            w.WriteLine("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
            w.WriteLine("<METADATA xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">");
            w.WriteLine("  <MDR-DVD>");
            w.WriteLine("    <MetadataExpires>2222-11-05T15:07:10.2829222+01:00</MetadataExpires>");
            w.WriteLine("    <version>5.0</version>");
            w.WriteLine("    <dvdTitle>" + dvdtitle + "</dvdTitle>");
            w.WriteLine("    <studio>" + studio + "</studio>");
            w.WriteLine("    <leadPerformer>" + leadperformer + "</leadPerformer>");
            w.WriteLine("    <director>" + director + "</director>");
            w.WriteLine("    <MPAARating>" + mpaarating + "</MPAARating>");
            w.WriteLine("    <language />");
            w.WriteLine("    <releaseDate>" + releasedate + "</releaseDate>");
            w.WriteLine("    <genre>" + genre + "</genre>");
            w.WriteLine("    <largeCoverParams />");
            w.WriteLine("    <smallCoverParams />");
            w.WriteLine("    <dataProvider>" + dataprovider + "</dataProvider>");
            w.WriteLine("    <duration>" + duration + "</duration>");
            w.WriteLine("    <title>");
            w.WriteLine("      <titleNum>1</titleNum>");
            w.WriteLine("      <titleTitle>" + dvdtitle + "</titleTitle>");
            w.WriteLine("      <studio>" + studio + "</studio>");
            w.WriteLine("      <director>" + director + "</director>");
            w.WriteLine("      <leadPerformer>" + leadperformer + "</leadPerformer>");
            w.WriteLine("      <MPAARating>" + mpaarating + "</MPAARating>");
            w.WriteLine("      <genre>" + genre + "</genre>");
            w.WriteLine("      <synopsis>" + synopsis + "</synopsis>");
            w.WriteLine("      <chapter>");
            w.WriteLine("        <chapterNum>1</chapterNum>");
            w.WriteLine("        <chapterTitle>" + dvdtitle + "</chapterTitle>");
            w.WriteLine("      </chapter>");
            w.WriteLine("    </title>");
            w.WriteLine("  </MDR-DVD>");
            w.WriteLine("  <NeedsAttribution>false</NeedsAttribution>");
            w.WriteLine("  <DvdId>" + dvdid + "</DvdId>");
            w.WriteLine("</METADATA>");
            w.Flush();
            w.Close();
        }
    }


}
